#!python
from math import log10

def raiseXtoYmodN(x, y, n):
    if y == 1:
        return x % n

    elif (y & 1) == 0:
        tmp = raiseXtoYmodN(x, y/2, n)
        return (tmp * tmp) % n

    else:
        tmp = raiseXtoYmodN(x, (y-1)/2, n)
        tmp = (tmp * tmp) % n
        tmp = (tmp * x) % n
        return tmp


print "\nDiffie-Hellman key exchange"
print "\nNote: Typical values for x and y range from 1024 to 4096 bits (%d to %d decimal digits)" %\
      (log10(pow(2, 1024)), log10(pow(2, 4096)))
print "      n(n-1)/2 must be prime and large, typically in the same range as x and y)"
print "      g must be primitive mod n, it is typically a single digit, say 3"

print "\nPick public values"

g = 3
n = 101
print "g: %(g)s, n: %(n)s" % locals()

x = 4
print "\nAlice chooses x: %(x)s" % locals()
X = raiseXtoYmodN(g, x, n)
print "calculates X: %(X)s and sends it to Bob" % locals()

y = 5
print "\nBob chooses y: %(y)s" % locals()
Y = raiseXtoYmodN(g, y, n)
print "calculates Y: %(Y)s and sends it to Alice" % locals()

print "\nAlice calculates kx with her x: %(x)s and the Y: %(Y)s Bob sent" % locals()
kx = raiseXtoYmodN(Y, x, n)
print "kx: %(kx)s" % locals()

print "\nBob calculates ky with his y: %(y)s and the X: %(X)s Alice sent" % locals()
ky = raiseXtoYmodN(X, y, n)
print "ky: %(ky)s" % locals()

print "x: %(x)s, y: %(y)s" % locals()

print "\nShowing in full"
kxy = pow(pow(g, x), y)
print "(g^x)^y: %s" % pow(pow(g, x), y)

kyx = pow(pow(g, y), x)
print "(g^y)^x: %(kyx)s" % locals()

k = kyx % n
print "kyx %% n: %(k)s" % locals()

print "\nNote that the order of exponenation is critical"
print "2^(3^4): %s" % pow(2, pow(3, 4))
print "2^(4^3): %s" % pow(2, pow(4, 3))
print "(2^3)^4: %s" % pow(pow(2, 3), 4)
print "(2^4)^3: %s" % pow(pow(2, 4), 3)
