#!python
from math import log10

def raiseXtoYmodN(x, y, n):
    if y == 1:
        return x % n

    elif (y & 1) == 0:
        tmp = raiseXtoYmodN(x, y/2, n)
        return (tmp * tmp) % n

    else:
        tmp = raiseXtoYmodN(x, (y-1)/2, n)
        tmp = (tmp * tmp) % n
        tmp = (tmp * x) % n
        return tmp

def GCD(a,b):
    while a:
        a, b = b%a, a
    return b

def extendedGCD(a, b):
    # ax + by = gcd(a, b)
    x,y, u,v = 0,1, 1,0
    while a != 0:
        q, r = b/a, b%a
        m, n = x-u*q, y-v*q
        b,a, x,y, u,v = a,r, u,v, m,n
    return x


print "\nRSA Key Pair Generation"

print "\nTo encrypt using the public key: c = (p^e) % n"
print "To decrypt using the public key: p = (c^d) % n"
print "Note: that integers are encrypted/decrypted"
print "      the integer to be encrypted must be smaller than n"
print "      typical values for n range from 1024 to 4096 bits (%d to %d decimal digits)" %\
      (log10(pow(2, 1024)), log10(pow(2, 4096)))

print "\nChoose the two prime values where p != q"
p = 17
q = 11
print "p: %(p)s, q: %(q)s" % locals()

print "\nCalculate n = p * q and phi = (p-1) * (q-1)"
n = p * q
phi = (p-1) * (q-1)
print "n: %(n)s, phi: %(phi)s" % locals()

print "\nChoose e such that 1 < d < phi"
print "   and e is relatively prime to phi (GCD(e, phi) = 1)"
e = 7
while (GCD(phi, 7) != 1):
    e = e + 2
print "e: %(e)s" % locals()

print "\nDetermine d such that 1 < e < phi"
print "      and ((e*d) % phi) == 1 (use Euclid's extended algorithm)"
d = extendedGCD(e, phi)
print "d: %(d)s" % locals()

print "\nPublic key  {e, n}: {%(e)2d, %(n)3d}" % locals()
print "Private key {d, n}: {%(d)2d, %(n)3d}" % locals()

print "\nLet the plain text be the number 88"
p = 88
print "p: %(p)s" % locals()

print "\nTo encrypt using the public key: c = (p^e) % n"
c = raiseXtoYmodN(p, e, n)
print "p^e: %s", pow(p, e)
print "c = (%(p)s^%(e)s) %% %(n)s" % locals()
print "c: %(c)s" % locals()

print "\nTo decrypt using the private key: p = (c^d) % n"
p = raiseXtoYmodN(c, d, n)
print "c^d: %s", pow(c, d)
print "p = (%(c)s^%(d)s) %% %(n)s" % locals()
print "p: %(p)s" % locals()


print "\nLarger but still much, much smaller than those used in practice"
n = 6012707
e = 3674911
d = 422191
print "n: %(n)s, e: %(e)s, d: %(d)s" % locals()
p1 = 5234673
c = raiseXtoYmodN(p1, e, n)
p2 = raiseXtoYmodN(c, d, n)
print "p1: %(p1)s, c: %(c)s, p2: %(p2)s" % locals()
